--[[

  Used to tune various Spirit FBL tuning parameters on a FrSky/opentx radio (as a telemetry script)
  Use Min/Max in Opentx to align Spirit display Values with Values from Spirit Settings software
  Assign new parameters:
	assign GV in Mixes and name the outputs
	add params block (+ increase variable maxpar)
	create sound files (2 times 100% volume)
	assign channels in the spirit unit
	Verify parameter range with spirit settings software and adjust with channel min/max values (output tab in opentx)
  script uses flight mode 0,2 and 4 (fmode) to store individual parameter values for different flight conditions (main rotor rpm) 
--]]



-- initialization

local function init()

 curpar = 	1			-- start with the first parameter
 maxpar = 	9			-- set to number of parameters
 adjust = 	0
 announce = 	0
 firstrun = 	1
 lcd.resetBacklightTimeout()		-- disable switching backlight off
 disable_sound = 1			-- disable parameter and value sounds  (0 = sound on, 1 = sound off)
 

end

local function sayparam(filename)
 if disable_sound == 0 then
  playFile(filename)
 end
end

local function sayvalue(number)
 if disable_sound == 0 then
  playNumber(number,0)
 end
end

local function curnum()								-- convert GV value to Spirit display value|1024)/2 = Umwandlung zu 0-1024 Skala ,  /128.125 =  1025 Werte geteilt durch 8(=Skala), +1 = Offset
 div = 101 / (params[curpar]["max"]-params[curpar]["min"]+1)
 offs = params[curpar]["min"]
 currentnumber = (((model.getGlobalVariable(params[curpar]["gvarindex"], fmode)+101)/2)/div)+offs
 return currentnumber
end

local function gvvalue(spvalue)							-- convert Spirit value to value for global variable
 oneunit = 201/((params[curpar]["max"]-params[curpar]["min"]+1))
 channelvalue = (oneunit * (spvalue-params[curpar]["min"]) + (0.5 * oneunit))-100.5
 return channelvalue
end

-- run

local function run(event)

-- === Set Up ===

 params = {}

  params[1] = {}
  params[1]["name"]		= "Pitch Pump Booster"
  params[1]["min"]		= 0
  params[1]["max"]		= 4
  params[1]["chgweight"]	= 1
  params[1]["soundfile"]	= "/SOUNDS/en/ppb.wav"
  params[1]["gvarindex"]	= 0					-- gvar1 = index 0
  params[1]["info1"]		= "Boosts collective pitch movements."
  params[1]["info2"]		= "Usually: 0"

  params[2] = {}
  params[2]["name"]		= "Flight Style"
  params[2]["min"]		= 1
  params[2]["max"]		= 8
  params[2]["chgweight"]	= 1
  params[2]["soundfile"]	= "/SOUNDS/en/flightst.wav"
  params[2]["gvarindex"]	= 1					-- gvar2 = index 1
  params[2]["info1"]		= "Decr: more constant/linear (robotic)."
  params[2]["info2"]		= "Incr: flybar like (decreased delay)"

  params[3] = {}
  params[3]["name"]		= "Pirouette Consistency"
  params[3]["min"]		= 50
  params[3]["max"]		= 250
  params[3]["chgweight"]	= 5
  params[3]["soundfile"]	= "/SOUNDS/en/pirocon.wav"
  params[3]["gvarindex"]	= 2					-- gvar3 = index 2
  params[3]["info1"]		= "specifies heading hold performance"
  params[3]["info2"]		= "Mostly between 155 and 180"

  params[4] = {}
  params[4]["name"]		= "Cyclic Feed Forward"
  params[4]["min"]		= 1
  params[4]["max"]		= 12
  params[4]["chgweight"]	= 1
  params[4]["soundfile"]	= "/SOUNDS/en/cff.wav"
  params[4]["gvarindex"]	= 3					-- gvar4 = index 3
  params[4]["info1"]		= "higher values: more crisp and"
  params[4]["info2"]		= "more sensitive cyclic feeling"

  params[5] = {}
  params[5]["name"]		= "Rudder Rotation Speed"
  params[5]["min"]		= 5
  params[5]["max"]		= 20
  params[5]["chgweight"]	= 1
  params[5]["soundfile"]	= "/SOUNDS/en/rrs.wav"
  params[5]["gvarindex"]	= 4					-- gvar5 = index 4
  params[5]["info1"]		= "Tail Rotation Rate"
  params[5]["info2"]		= ""

  params[6] = {}
  params[6]["name"]		= "Cyclic Rotation Speed"
  params[6]["min"]		= 5
  params[6]["max"]		= 16
  params[6]["chgweight"]	= 1
  params[6]["soundfile"]	= "/SOUNDS/en/crs.wav"
  params[6]["gvarindex"]	= 5					-- gvar6 = index 5
  params[6]["info1"]		= "Cyclic Rotation Rate"
  params[6]["info2"]		= ""

  params[7] = {}
  params[7]["name"]		= "Elevator Filter"
  params[7]["min"]		= 0
  params[7]["max"]		= 4
  params[7]["chgweight"]	= 1
  params[7]["soundfile"]	= "/SOUNDS/en/ef.wav"
  params[7]["gvarindex"]	= 6					-- gvar7 = index 6
  params[7]["info1"]		= "compensates elevator bouncing during aggressive maneuvers"
  params[7]["info2"]		= "higher=more compensation. too high: soft elev. feel"

  params[8] = {}
  params[8]["name"]		= "Head Gain"
  params[8]["min"]		= 20
  params[8]["max"]		= 100
  params[8]["chgweight"]	= 5
  params[8]["soundfile"]	= "/SOUNDS/en/hgain.wav"
  params[8]["gvarindex"]	= 7					-- gvar8 = index 7
  params[8]["info1"]		= "lower: feels more natural"
  params[8]["info2"]		= "needs to be low for fast PTT"

  params[9] = {}
  params[9]["name"]		= "Tail Gain"
  params[9]["min"]		= -100
  params[9]["max"]		= 100
  params[9]["chgweight"]	= 5
  params[9]["soundfile"]	= "/SOUNDS/en/tgain.wav"
  params[9]["gvarindex"]	= 8					-- gvar9 = index 8
  params[9]["info1"]		= "Head-Lock mode: 1% to 100%"
  params[9]["info2"]		= "Normal(Rate) or special function: -100% to 0%"

--[[
  params[4] = {}
  params[4]["name"]		= "Anti Gravity"
  params[4]["min"]		= 0
  params[4]["max"]		= 50
  params[4]["chgweight"]	= 5
  params[4]["soundfile"]	= "/SOUNDS/en/antig.wav"
  params[4]["gvarindex"]	= 3				-- gvar4 = index 3
  params[4]["info1"]		= "Degree of collective pitch precomp."
  params[4]["info2"]		= "Also called Lightness"					--]]



-- === Output ===

 lcd.clear()

--set current flight condition
 swe = getValue('se')
-- lcd.drawNumber(180,43,swe, SMLSIZE)
 if swe == -1024 then		-- se is down / low rpm
  fmode = 0
 end
 if swe == 0 then		-- se is middle / autorotation rpm
  fmode = 2
 end
 if swe == 1024 then		-- se is up / high rpm
  fmode = 4
 end


 if announce == 1 then		-- announce value if it has changed
  sayvalue(math.floor(curnum()))
  announce = 0
 end

 if firstrun == 1 then 		-- announce parameter and value at first start
  sayparam(params[curpar]["soundfile"])
--  sayvalue(math.floor(curnum()))
  firstrun = 0
 end

 --toggle parameters
 if event == EVT_PLUS_FIRST and adjust == 0 then
  curpar = curpar + 1
  if params[curpar] == nil then
   curpar = 1
  end
  sayparam(params[curpar]["soundfile"])
--  sayvalue(math.floor(curnum()))
 end
 if event == EVT_MINUS_FIRST and adjust == 0 then
  curpar = curpar - 1
  if params[curpar] == nil then
   curpar = maxpar
  end
  sayparam(params[curpar]["soundfile"])
--  sayvalue(math.floor(curnum()))
 end

-- lcd.drawText(130,     43, "gvvalue5: "..gvvalue(5), SMLSIZE)
--lcd.drawText(110,     43, "getValuech14: "..getValue("ch14"), SMLSIZE)


 --adjust values
  if event == EVT_PLUS_FIRST and adjust == 1 then
   if math.floor(curnum())+params[curpar]["chgweight"] <= params[curpar]["max"] then
    model.setGlobalVariable(params[curpar]["gvarindex"], fmode, gvvalue(math.floor(curnum())+params[curpar]["chgweight"]))
   end
   announce=1
  end
  if event == EVT_MINUS_FIRST and adjust == 1 then
   if math.floor(curnum())-params[curpar]["chgweight"] >= params[curpar]["min"] then
-- lcd.drawText(130,     43, "gvvalue: "..gvvalue(math.floor(curnum())-params[curpar]["chgweight"]), SMLSIZE)
    model.setGlobalVariable(params[curpar]["gvarindex"], fmode, gvvalue(math.floor(curnum())-params[curpar]["chgweight"]))
   end
   announce=1
  end

 -- toggle beetween parameter switch and parameter change mode
 if event == EVT_ENTER_FIRST then
  if adjust == 0 then
   adjust=1
   playFile("v mode.wav")
   sayvalue(math.floor(curnum()))
  else
   adjust=0
   playFile("p mode.wav")
  end
 end

-- lcd.drawText (180,13,params[curpar]["name"],SMLSIZE)
 lcd.drawText(3,25,"Value: "..math.floor(curnum()).."                        Min: "..params[curpar]["min"].."  Max: "..params[curpar]["max"], 0)
 if adjust == 0 then
  lcd.drawText (3,3,"P mode. "..params[curpar]["info1"],SMLSIZE)
 else
 lcd.drawText (3,3,"V mode. "..params[curpar]["info1"],SMLSIZE)
 end

-- draw info text
 lcd.drawText (3,14,params[curpar]["info2"],SMLSIZE)

 -- draw bar
 lcd.drawText(10,43,params[curpar]["name"], MIDSIZE)
 lcd.drawGauge(1, 35, 210, 25, curnum()-params[curpar]["min"] , params[curpar]["max"]-params[curpar]["min"])

 if event == EVT_PAGE_BREAK then
  firstrun = 1
 end

end

return { run=run, init=init }
